﻿// Unity built-in shader source. Copyright (c) 2016 Unity Technologies. MIT license (see license.txt)

Shader "Sprites/HSV" {
    Properties {
        [PerRendererData] _MainTex ("Sprite Texture", 2D) = "white" {}
        _Color ("Tint", Color) = (1,1,1,1)
        _Hue ("Hue", Range(0, 1)) = 0
        _Saturation ("Saturation", Range(0, 1)) = 0
        _Value ("Value", Range(-1,1)) = 0
        _PreserveColor("Preserve Color", Color) = (1,1,1,1)
        _PreserveTolerance("Preserve Tolerance", Range(0, 1)) = 0.1
        _OutlineColor("Outline Color", Color) = (1,1,1,1)
        _OutlineSize("Outline Size", int) = 0
        [MaterialToggle] PixelSnap ("Pixel snap", Float) = 0
        [HideInInspector] _RendererColor ("RendererColor", Color) = (1,1,1,1)
        [HideInInspector] _Flip ("Flip", Vector) = (1,1,1,1)
        [PerRendererData] _AlphaTex ("External Alpha", 2D) = "white" {}
        [PerRendererData] _EnableExternalAlpha ("Enable External Alpha", Float) = 0
    }

    SubShader {
        Tags { 
            "Queue"="Transparent" 
            "IgnoreProjector"="True" 
            "RenderType"="Transparent" 
            "PreviewType"="Plane"
            "CanUseSpriteAtlas"="True"
        }

        Cull Off
        Lighting Off
        ZWrite Off
        Blend One OneMinusSrcAlpha

        Pass {
        CGPROGRAM
            #pragma vertex SpriteVert
            #pragma fragment SpriteFragGreyscale
            #pragma target 2.0
            #pragma multi_compile_instancing
            #pragma multi_compile _ PIXELSNAP_ON
            #pragma multi_compile _ ETC1_EXTERNAL_ALPHA
            #include "UnitySprites.cginc"

            uniform float _Saturation;
            uniform float _Hue;
            uniform float _Value;
            fixed4 _OutlineColor;
            fixed4 _PreserveColor;
            int _OutlineSize;
            float4 _MainTex_TexelSize;
            float _PreserveTolerance;

            float3 rgb2hsv(float3 RGB) {
                float4 k = float4(0.0, -1.0/3.0, 2.0/3.0, -1.0);
                float4 p = RGB.g < RGB.b ? float4(RGB.b, RGB.g, k.w, k.z) : float4(RGB.gb, k.xy);
                float4 q = RGB.r < p.x   ? float4(p.x, p.y, p.w, RGB.r) : float4(RGB.r, p.yzx);
                float d = q.x - min(q.w, q.y);
                float e = 1.0e-10;
                return float3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
            }

            float3 hsv2rgb(float3 HSV) {
                float4 k = float4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
                float3 p = abs(frac(HSV.xxx + k.xyz) * 6.0 - k.www);
                return HSV.z * lerp(k.xxx, clamp(p - k.xxx, 0.0, 1.0), HSV.y);
            }

            fixed4 SpriteFragGreyscale (v2f IN) : COLOR {
                fixed4 c = SampleSpriteTexture (IN.texcoord);
                float4 delta = c.rgba - _PreserveColor.rgba;

                if(dot(delta, delta) > _PreserveTolerance){
                    c.rgb = rgb2hsv(c.rgb);
                    c.r += _Hue;
                    c.rgb = hsv2rgb(c.rgb);
                    c.rgb = lerp(c.rgb, dot(c.rgb, float3(0.3, 0.59, 0.11)), 1 - _Saturation) * IN.color;
                }
                c.rgb = clamp(0, 1, c.rgb + _Value);
                c.rgb *= c.a;

                for (int i = 1; i < _OutlineSize + 1; i++) {
                    fixed4 currentPixel = tex2D(_MainTex, IN.texcoord + fixed2(0, 0));
                    fixed4 pixelUp = tex2D(_MainTex, IN.texcoord + fixed2(0, i * _MainTex_TexelSize.y));
                    fixed4 pixelDown = tex2D(_MainTex, IN.texcoord - fixed2(0,i *  _MainTex_TexelSize.y));
                    fixed4 pixelRight = tex2D(_MainTex, IN.texcoord + fixed2(i * _MainTex_TexelSize.x, 0));
                    fixed4 pixelLeft = tex2D(_MainTex, IN.texcoord - fixed2(i * _MainTex_TexelSize.x, 0));
 
                    if (currentPixel.a == 0) {
                        if (pixelUp.a > 0 || pixelDown.a > 0 || pixelRight.a > 0 || pixelLeft.a > 0)
                            c.rgba = fixed4(1, 1, 1, 1) * _OutlineColor;
                    }
                }

                return c;
            }

        ENDCG
        }
    }
}
