﻿using System.Reflection;
using UnityEngine;
using ByteSprite.DependencyInjection.Container;
using ByteSprite.DependencyInjection.Installer;

namespace ByteSprite.DependencyInjection {
    public static class Injector {
		static IInjectionContainer container;

        static readonly BindingFlags Flags = BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public;

        public static void Inject(object @object){
			if(container == null) {
				var installer = Object.FindObjectOfType<MonoInjectionInstaller> ();
				installer.SetBindings();
				container = installer.Container;
			}

            InjectFields(@object);
            InjectProperties(@object);
            InjectMethods(@object);
        }

        static void InjectFields(object @object){
            var type = @object.GetType();

            foreach (var field in type.GetFields(Flags)) {
                foreach (var attribute in field.GetCustomAttributes(typeof(InjectAttribute), false)) {
					var value = container?.GetDependency(field.FieldType, ((InjectAttribute)attribute).Id);

                    if (value == null) {
						NoBindingError (type.Name, field.Name);
						continue;
                    }

                    field.SetValue(@object, value);
                }
            }
        }

        static void InjectProperties(object @object){
            var type = @object.GetType();

            foreach (var property in type.GetProperties(Flags)) {
                foreach (var attribute in property.GetCustomAttributes(typeof(InjectAttribute), false)) {
					var value = container?.GetDependency(property.PropertyType, ((InjectAttribute)attribute).Id);

                    if (value == null) {
						NoBindingError (type.Name, property.Name);
                        continue;
                    }

                    property.SetValue(@object, value);
                }
            }
        }

        static void InjectMethods(object @object){
            var type = @object.GetType();

            foreach (var method in type.GetMethods(Flags)){
                foreach (var attribute in method.GetCustomAttributes(typeof(InjectAttribute), false)) {
                    var paramaters = method.GetParameters();
                    var values = new object[paramaters.Length];

                    for(int i = 0; i < paramaters.Length; i++) {
						var value = container?.GetDependency(paramaters[i].ParameterType, ((InjectAttribute)attribute).Id);

                        if (value == null) {
							NoBindingError(type.Name, paramaters[i].Name);
                            continue;
                        }

                        values[i] = value;
                    }

                    method.Invoke(@object, values);
                }
            }
        }

		static void NoBindingError(string typeName, string injectableName){
			Debug.LogError ($"No binding found for <{typeName}, {injectableName}>.");
		}
    }
}