﻿using System.Collections;
using UnityEngine;

namespace Plants {
	public class AnimationManager : MonoBehaviour {
		[SerializeField]
		AnimationCurve curve;

	    int animationCount;
	    public bool IsAnimating {
	        get { return animationCount > 0; }

	        set {
	            animationCount += value ? 1 : -1;
	            animationCount = Mathf.Max(0, animationCount);
	        }
	    }

		public void AnimateTranslate(Transform moving, Vector3 endPosition, float duration, System.Action callback = null) {
			StartCoroutine(AnimateTranslateOverTime(moving, endPosition, duration, callback));
	    }

		public void AnimateFade(SpriteRenderer renderer, float startValue, float endValue, float duration) {
			StartCoroutine(AnimateFadeOverTime(renderer, startValue, endValue, duration));
		}

		IEnumerator AnimateTranslateOverTime(Transform moving, Vector3 endPosition, float duration, System.Action callback){
	        IsAnimating = true;
	        var startPosition = moving.transform.position;
	        float elapsed = 0f;

	        while(elapsed < 1f){
	            elapsed = Mathf.Clamp01(elapsed + (Time.deltaTime / duration));

				var curved = curve.Evaluate(elapsed);

				moving.transform.position = Vector3.Lerp(startPosition, endPosition, curved);
	            yield return null;
	        }

	        moving.transform.position = endPosition;
			callback?.Invoke();
	        IsAnimating = false;
	    }


		IEnumerator AnimateFadeOverTime (SpriteRenderer renderer, float startValue, float endValue, float duration){
			IsAnimating = true;
			float elapsed = 0f;

			while (elapsed < 1f) {
				elapsed = Mathf.Clamp01 (elapsed + (Time.deltaTime / duration));

				var curved = curve.Evaluate (elapsed);

				//var alpha = Mathf.Lerp(startValue, endValue, curved);

				renderer.color = Color.Lerp(renderer.color, Color.black, 0.2f);
				yield return null;
			}

			IsAnimating = false;
		}
	}
}