﻿using UnityEngine;
using ByteSprite.DependencyInjection;

namespace Plants {
	public class PlantManager : MonoBehaviour {
		[Inject]
		AnimationManager animationManager;

		void Awake(){
			Injector.Inject(this);
		}

		public void MoveOccupant(ITile tileA, ITile tileB) {
			if(tileA == null || tileB == null)
				return;

			var occupant = tileA?.Occupant;
			tileA.Occupant = tileB?.Occupant;
			tileB.Occupant = occupant;

			if (tileA.Occupant != null)
				animationManager.AnimateTranslate (tileA.Occupant.Transform, tileA.Position, 0.4f);

			if(tileB.Occupant != null)
				animationManager.AnimateTranslate (tileB.Occupant.Transform, tileB.Position, 0.4f);
		}

		public void HoverPlant(Vector3? position, Plant hover){
			var plants = FindObjectsOfType<Plant>();
			foreach(var plant in plants){
				var renderer = plant.GetComponent<SpriteRenderer> ();
				if((position == null) || hover == plant){
					renderer.color = Color.white;
				} else {
					float distance = Vector3.Distance(plant.transform.position, position ?? Vector3.zero);
					renderer.color = new Color(1f, 1f, 1f, Mathf.Clamp01(distance/4f));
				}
			}
		}
	}
}