﻿using UnityEngine;

namespace Plants {
	[RequireComponent(typeof(SpriteRenderer))]
	public class Plant : MonoBehaviour, IPlant {
		new SpriteRenderer renderer;

		[SerializeField]
		new string name;

		[SerializeField]
		EWater requiredWater;

		[SerializeField]
		ELighting requiredLight;

		[SerializeField]
		EOxygen oxygenProduced;

		[SerializeField]
		EGrowth growthRate;

		[SerializeField]
		ELocation location;

		[SerializeField]
		Sprite juvi, mature, dead, icon;

		ParticleSystem waterParticles;

		float damage;
		public float Damage { get { return damage; }
			set {
				damage = Mathf.Clamp01(value);
				if(damage >= 1f && IsAlive){
					Die();
				}
			}
		}

		public Sprite Icon { get { return icon; } }

		public PlantGlow PlantGlow { get; set; }

		public string Name { get { return name; } private set { name = value; } }

		public Transform Transform { get { return transform; } }

		public EWater RequiredWater { get { return requiredWater; } private set { requiredWater = value; } }
		public ELighting RequiredLight { get { return requiredLight; } private set { requiredLight = value; } }
		public EOxygen OxygenProduced { get { return oxygenProduced; } private set { oxygenProduced = value; } }
		public EGrowth GrowthRate { get { return growthRate; } private set { growthRate = value; } }
		public ELocation Location { get { return location; } private set { location = value; } }

		public Sprite Juvi { get { return juvi; } private set { juvi = value; } }
		public Sprite Mature { get { return mature; } private set { mature = value; } }
		public Sprite Dead { get { return dead; } private set { dead = value; } }

		public float CurrentWater { get; private set; }
		public float Age { get; private set; }

		bool isAlive = true;
		public bool IsAlive {
			get{ return isAlive; }
			private set{
				isAlive = value;
				OxygenProduced = EOxygen.None;
				PlantGlow.PlantRenderer.enabled = false;
				Age = 0f;
				GetComponent<Animator>().speed = 0f;
				waterParticles.emissionRate = 0f;
				CurrentWater = 0f;
			}
		}

		public void AbsorbWater() {
			float waterLoss = 0f;

			switch (RequiredWater) {
				case EWater.Low: waterLoss = 0.333f; break;
				case EWater.Medium: waterLoss = 0.666f; break;
				case EWater.High: waterLoss = 1f; break;
			}

			CurrentWater = Mathf.Clamp01(CurrentWater - waterLoss);
		}

		public void Water() {
			if(IsAlive)
				CurrentWater = Mathf.Clamp01(CurrentWater + 0.666f);
			waterParticles.emissionRate = 10f;
		}

		public void Grow () {
			if(!IsAlive){
				Age = 0f;
				return;
			}

			float growth = 0f;

			switch(GrowthRate){
					case EGrowth.Low : growth = 0.01f; break;
					case EGrowth.Medium : growth = 0.02f; break;
					case EGrowth.High : growth = 0.03f; break;
			}

			Age = Mathf.Clamp01(Age + growth);

			if(Age > 0.5f){
				UpdateSprite(mature);
			}
		}

		public void Die() {
			IsAlive = false;
			UpdateSprite(Dead);
		}

		void Awake() {
			renderer = GetComponent<SpriteRenderer>();
			waterParticles = GetComponentInChildren<ParticleSystem>();
			waterParticles.emissionRate = 0f;
			growTime = growTimeMax;
		}

		void Start() {
			UpdateSprite (Juvi);
			growTime = growTimeMax;
		}

		float growTime;
		float growTimeMax = 2.5f;

		void Update() {
			growTime -= Time.deltaTime;

			if(CurrentWater <= 0.9f)
				waterParticles.emissionRate -= Time.deltaTime * 10f;

			if(CurrentWater > 0.9f){
				waterParticles.emissionRate = 20f;
			}

			if(growTime <= 0f){
				growTime = growTimeMax;
				Grow();
				DamageHealPlant();
			}

			float oxygen = 0f;

			switch(OxygenProduced){
			case EOxygen.Low:
				oxygen = 2f;
				break;
			case EOxygen.Medium:
				oxygen = 2.5f;
				break;
			case EOxygen.High:
				oxygen = 3f;
				break;
			}

			oxygen *= Age > 0.5f ? 1f : 0.5f;

			PlantGlow.Glow.localScale = Vector3.one * oxygen;
		}

		void UpdateSprite(Sprite sprite){
			renderer.sprite = sprite;
			PlantGlow.PlantRenderer.sprite = sprite;
		}

		void DamageHealPlant () {
			CurrentWater -= 0.0333f;

			if(CurrentWater < 0.1f || CurrentWater > 0.9f)
				Damage += 0.05f;

			var tiles = FindObjectsOfType<Tile> ();
			foreach (Tile tile in tiles) {
				if ((Plant)tile.Occupant != this)
					continue;

				var occupant = tile.Occupant;
				var lightingDifference = Mathf.Abs(RequiredLight - tile.Lighting) - 0.5f;
				Damage += 0.05f * lightingDifference;
			}
		}
	}
}